#!/usr/bin/python

import sys, socket, select, marshal, time
from types import FunctionType, CodeType

status = {
	"OK" : 1,
	"NO_FLAG" : 2,
	"MUMBLE" : 3,
	"DOWN" : 4,
	"INTERNAL_ERROR" : 10
}

class ServiceError(Exception):
	def __init__(self, msg, status_code):
		Exception.__init__(self)
		self.msg = msg
		self.status = status[status_code]

	def __str__(self):
		return self.msg

class NoFlag(ServiceError):
	def __init__(self, msg):
		ServiceError.__init__(self, msg, "NO_FLAG")

class ServiceCorrupt(ServiceError):
	def __init__(self, msg):
		ServiceError.__init__(self, msg, "MUMBLE")

class ServiceDown(ServiceError):
	def __init__(self, msg):
		ServiceError.__init__(self, msg, "DOWN")

class InternalError(ServiceError):
	def __init__(self, msg):
		ServiceError.__init__(self, msg, "INTERNAL_ERROR")

def log(msg):
	s = "[" + time.ctime() + "] " + msg + "\n"
	try:
		f = open("log.txt", "a")
		f.write(s)
		f.close()
	except:
		raise InternalError("cannot open log file")

class Checker:
	def __init__(self, ip):
		self.ip = ip
		try: self.sock = socket.socket()
		except: raise InternalError("cannot create socket")
		try: self.sock.connect((ip, 0xDDDD))
		except: raise ServiceDown("cannot connect to service")

	def __del__(self):
		try: self.sock.shutdown(socket.SHUT_RDWR)
		except: pass
		self.sock.close()


	def procCAPTCHA(self):
		def filter(co):
			code = co.co_code
			newcode = ""
			ops1 = "\x01\x02\x03\x04\x05\x0A\x0B\x0F\x14\x17\x18\x40\x41\x42\x53"
			ops2 = "\x64\x7C\x7D"
			i = 0
			while i < len(code):
				if code[i] in ops1: 
					newcode += code[i]
				elif code[i] in ops2:
					newcode += code[i:i+3]
					i += 2
				i += 1
			return CodeType(3, 3, co.co_stacksize, 0x41, newcode, (9487, 3129, 7828), (), ('a', 'b', 'c'), 'gen.py', 'test', 1, "\x01" * 1000, (), ())

		def makeFunc(co):
			return FunctionType(co, {})

		def readExactLen(sock, length, timeout):
			r = self.sock.recv(length)
			while len(r) < length:
				ready = select.select([self.sock], [], [], timeout)[0]
				if not ready:
					raise ServiceCorrupt("couldn't get enough data...")
				else:
					r += self.sock.recv(length - len(r))
			return r

		try:
			greeting = readExactLen(self.sock, 2762, 0.5)
			if len(greeting) != 2762:
				print "alarma!!!"
			co = self.sock.recv(4096)
			ready = select.select([self.sock], [], [], 0.5)[0]
			while ready:
				print "more data..."
				co += self.sock.recv(4096)
				ready = select.select([self.sock], [], [], 0.5)[0]
			test = makeFunc(filter(marshal.loads(co)))
			res = test(1981, 3223, 1298)
			self.sock.send(str(res))
			self.sock.recv(100000)
		except:
			import traceback
			traceback.print_exc()
			raise ServiceCorrupt("captcha is broken")

	def put(self, id, flag):
		self.procCAPTCHA()
		putCO = CodeType(1, 1, 1, 0x41, "\xaf\x64\x02\x00\x53", (id, flag, None), (), ('client',), 'checker.py', 'put', 1, "\x01" * 1000, (), ())
		try:
			self.sock.send(marshal.dumps(putCO))
		except:
			raise ServiceCorrupt("cannot put the flag")
		# timeout before next get
		time.sleep(1)

	def get(self, id, flag):
		self.procCAPTCHA()
		getCO = CodeType(1, 1, 1, 0x41, "\xb0\x64\x01\x00\x53", (id, None), (), ('client',), 'checker.py', 'get', 1, "\x01" * 1000, (), ())
		try:
			self.sock.send(marshal.dumps(getCO))
			res = self.sock.recv(100000)
		except:
			raise ServiceCorrupt("cannot get the flag")
		if res != flag:
			#log(">>> res = " + str(res))
			#log(">>> flag = " + str(flag))
			raise NoFlag("bad flag")

	def check(self):
		self.procCAPTCHA()
		checkCO1 = CodeType(1, 1, 1, 0x41, "\xb1\x64\x00\x00\x53", (None,), (), ('client',), 'checker.py', 'check', 1, "\x01" * 1000, (), ()) 
		checkCO2 = CodeType(1, 1, 1, 0x41, "\xb2\x64\x00\x00\x53", (None,), (), ('client',), 'checker.py', 'check', 1, "\x01" * 1000, (), ()) 
		try:
			self.sock.send(marshal.dumps(checkCO1))
			res = self.sock.recv(100000)
		except:
			raise ServiceCorrupt("broken functionality")
		if not res:
			raise ServiceCorrupt("could not get bender quote")
		
		try:
			self.sock.send(marshal.dumps(checkCO2))
			res = self.sock.recv(100000)
		except:
			raise ServiceCorrupt("broken functionality")
		if not res:
			raise ServiceCorrupt("could not get a lovely pic")

if __name__ == '__main__':
	try:
		if len(sys.argv) < 3: raise InternalError("bad number of parameters were passed to checker")
		method = sys.argv[1]
		ip = sys.argv[2]
		if method not in ('check', 'put', 'get'): raise InternalError("unknown method: " + method)
		checker = Checker(ip)
		if method == 'check':
			if len(sys.argv) != 3: raise InternalError("wrong number of parameters for 'check' method")
			checker.check()
		elif method == 'put':
			if len(sys.argv) != 5: raise InternalError("wrong number of parameters for 'put' method")
			id, flag = sys.argv[3:]
			checker.put(id, flag)
		elif method == 'get':
			if len(sys.argv) != 5: raise InternalError("wrong number of parameters for 'get' method")
			id, flag = sys.argv[3:]
			checker.get(id, flag)
		del checker
	except ServiceError, e:
		log(" ".join(sys.argv) + ": " + str(e) + " (" + str(e.status) + ")")
		print e
		sys.exit(e.status)
	else:
		log(" ".join(sys.argv) + ": OK (" + str(status["OK"]) + ")")
		sys.exit(status["OK"])
